@@ -532,7 +532,7 @@ hostx_get_server_depth (EphyrScreenInfo screen)
 {
   struct EphyrHostScreen *host_screen = host_screen_from_screen_info (screen);
 
-  return (host_screen ? host_screen->server_depth : 0);
+  return host_screen ? host_screen->server_depth : 0;
 }
 
 void
@@ -669,11 +669,8 @@ hostx_screen_init (EphyrScreenInfo screen,
 	}
       else
 	{
-	  if (host_screen->ximg->data) 
-	    {
-	      free(host_screen->ximg->data);
-	      host_screen->ximg->data = NULL;
-	    } 
+	  free(host_screen->ximg->data);
+	  host_screen->ximg->data = NULL;
 
 	  XDestroyImage(host_screen->ximg);
 	}
@@ -1160,10 +1157,8 @@ out:
         XFree (visuals) ;
         visuals = NULL;
     }
-    if (host_visuals) {
-        free (host_visuals) ;
-        host_visuals = NULL;
-    }
+    free(host_visuals);
+    host_visuals = NULL;
     EPHYR_LOG ("leave\n") ;
     return is_ok ;
 
@@ -1292,10 +1287,8 @@ hostx_set_window_bounding_rectangles (int a_window,
                              rects, a_num_rects, ShapeSet, YXBanded) ;
     is_ok = TRUE ;
 
-    if (rects) {
-        free (rects) ;
-        rects = NULL ;
-    }
+    free(rects);
+    rects = NULL;
     EPHYR_LOG ("leave\n") ;
     return is_ok;
 }
@@ -1329,10 +1322,8 @@ hostx_set_window_clipping_rectangles (int a_window,
                              rects, a_num_rects, ShapeSet, YXBanded) ;
     is_ok = TRUE ;
 
-    if (rects) {
-        free (rects) ;
-        rects = NULL ;
-    }
+    free(rects);
+    rects = NULL;
     EPHYR_LOG ("leave\n") ;
     return is_ok;
 }
